#include "Pedigree.h"
#include "Parameters.h"
#include "StringArray.h"
#include "Error.h"
#include <iostream>
#include "Vcf2Map.h"

using namespace std;

int main(int argc, char* argv[])
{
  String pedFile, datFile;
  String mapFile("/scratch/cgg/lib13/db/hapmap/genetic_map_GRCh37_chr1.txt");
  String includeListFile("/scratch/cgg/Public/hg19/1000G.SNV.clean.MAF0.05.tbl.gz");
  String inVCF, outMapFile;

  int winSize = 100;
  double maxR2 = 0.1;
  double minMAF = 0.1;
  double minDP = 0.0;
  double maxDP = -1.0;
  double maxMissingRate = 0.05;
  bool ignoreMissing = false;

  bool useEUR_AF4Filter = false;
  bool useASN_AF4Filter = false;
  bool useAFR_AF4Filter = false;
  bool useALL_AF4Filter = false;
  bool useEUR_AF4Missing = false;
  bool useASN_AF4Missing = false;
  bool useAFR_AF4Missing = false;
  bool useALL_AF4Missing = false;

  ParameterList pl;

  BEGIN_LONG_PARAMETERS(longParameters)
   LONG_PARAMETER_GROUP("Input")
    LONG_STRINGPARAMETER("vcf", &inVCF)
    LONG_STRINGPARAMETER("ped", &pedFile)
    LONG_STRINGPARAMETER("map", &mapFile)
    LONG_STRINGPARAMETER("include_list", &includeListFile)
   LONG_PARAMETER_GROUP("Output")
    LONG_STRINGPARAMETER("out_map", &outMapFile)
   LONG_PARAMETER_GROUP("Variant filter")
    LONG_DOUBLEPARAMETER("min_maf", &minMAF)
    LONG_DOUBLEPARAMETER("min_avg_dp", &minDP)
    LONG_DOUBLEPARAMETER("max_avg_dp", &maxDP)
    LONG_DOUBLEPARAMETER("max_missing_rate", &maxMissingRate)
   LONG_PARAMETER_GROUP("LD pruning")
    LONG_INTPARAMETER("win_size", &winSize)
    LONG_DOUBLEPARAMETER("max_r2", &maxR2)
    LONG_PARAMETER("ignore_missing", &ignoreMissing)
/*   LONG_PARAMETER_GROUP("Using 1000G MAF to filter variants")
    LONG_PARAMETER("EUR_4filter", &useEUR_AF4Filter)
    LONG_PARAMETER("ASN_4filter", &useASN_AF4Filter)
    LONG_PARAMETER("AFR_4filter", &useAFR_AF4Filter)
    LONG_PARAMETER("ALL_4filter", &useALL_AF4Filter)
   LONG_PARAMETER_GROUP("Using 1000G MAF to impute missing genotype")
    LONG_PARAMETER("EUR_4missing", &useEUR_AF4Missing)
    LONG_PARAMETER("ASN_4missing", &useASN_AF4Missing)
    LONG_PARAMETER("AFR_4missing", &useAFR_AF4Missing)
    LONG_PARAMETER("ALL_4missing", &useALL_AF4Missing)
*/

  END_LONG_PARAMETERS();

  pl.Add(new LongParameters("", longParameters));

  pl.Read(argc, argv);

  pl.Status();

  if(inVCF.Length()==0)
    error("VCF file not provided for input!\n");

  IFILE datFH = ifopen(datFile, "r");
  IFILE pedFH = ifopen(pedFile, "r");
  IFILE mapFH = ifopen(mapFile, "r");
  IFILE incFH = ifopen(includeListFile, "r");
  IFILE vcfFH = ifopen(inVCF, "r");

  FILE *outMap = fopen(outMapFile.c_str(), "w");

  if(datFile.Length()>0 && datFH==NULL)
    error("datFile open for input failed!\n");
  if(pedFile.Length()>0 && pedFH==NULL)
    error("pedFile open for input failed!\n");
  if(mapFile.Length()>0 && mapFH==NULL)
    error("mapFile open for input failed!\n");
  if(vcfFH==NULL)
    error("Input VCF file open failed!\n");

  if(outMap==NULL) error("Open map file for output failed!\n");

  Pedigree ped;

  if(pedFile.Length()>0) {
//    ped.Prepare(datFH);
    ped.Load(pedFH);
  }

  Vcf2Map vcf2map;

  vcf2map.SetWinSize(winSize);
  vcf2map.SetMaxR2(maxR2);
  vcf2map.SetMinMAF(minMAF);
  vcf2map.SetMinAvgDP(minDP);
  vcf2map.SetMaxAvgDP(maxDP);
  vcf2map.SetMaxMissingRate(maxMissingRate);
  vcf2map.SetOutMapFile(outMap);

  if(pedFile.Length()>0)
	vcf2map.GetFounders(ped);

  if(includeListFile.Length()>0)
	vcf2map.LoadIncludeList(incFH);

  if(mapFile.Length()>0)
	vcf2map.LoadMapFile(mapFH);

  vcf2map.ProcessVCF(std::string(inVCF.c_str()));

}
